@@ -1,5 +1,8 @@
 @BOTTOM@
 
+/* Do we have getaddrinfo()? */
+#undef HAVE_GETADDRINFO_PROTO
+
 /* Do we have POSIX signals? */
 #undef HAVE_SIGACTION
 
@@ -39,4 +42,4 @@
 #undef PACKAGE
 #undef VERSION
 
-/* $Id: acconfig.h,v 4.2 2001/03/02 20:47:47 bortz Exp $ */
+/* $Id: acconfig.h,v 4.3 2001/03/19 09:38:32 bortz Exp $ */
