@@ -1,12 +1,13 @@
 /*
- * echoping : uses the TCP echo service to measure (roughly) response times.
+ * echoping : uses the TCP echo (or other TCP or UDP protocols)
+ * service to measure (roughly) response times.
  * 
  * Written by Stephane Bortzmeyer <bortz@users.sourceforge.net>. See
- * the AUTHORS file for other contributors.
+ * the AUTHORS file for other contributors. 
  * 
- * $Id: echoping.c,v 4.9 2001/03/06 16:46:00 bortz Exp $
- * 
- */
+ * $Id: echoping.c,v 4.25 2002/11/04 15:21:21 bortz Exp $
+ *
+ *  */
 
 
 char *progname;
@@ -24,16 +25,16 @@ unsigned short timeout_flag;
 /* Global variables for main and printstats */
 
 int return_code = 0;
+int rc;
 unsigned int number = 1;
-struct timeval max, min, total, median, temp;
+struct timeval max, min, total, median, stddev, temp;
+struct timeval conntv, connectedtv, sendtv, recvtv;
 unsigned int successes, attempts = 0;
 unsigned int size = DEFLINE;
 unsigned int j = 0;
-struct result
-{
-  unsigned short valid;
-  struct timeval timevalue;
-};
+
+int family = PF_UNSPEC;
+
 struct result results[MAXNUMBER];
 struct timeval good_results[MAXNUMBER];
 extern int tvcmp ();
@@ -46,22 +47,22 @@ main (argc, argv)
   extern char *optarg;
   extern int optind;
 
-  signed char ch;
+  int result;
 
   int sockfd;
+  struct addrinfo hints, *res;
+  int error;
+  char hbuf[NI_MAXHOST], pbuf[NI_MAXSERV];
+#ifdef NI_WITHSCOPEID
+  const int niflags = NI_NUMERICHOST | NI_NUMERICSERV | NI_WITHSCOPEID;
+#else
+  const int niflags = NI_NUMERICHOST | NI_NUMERICSERV;
+#endif
+
   FILE *files = NULL;
   CHANNEL channel;
-  struct hostent *hostptr;
-  struct sockaddr_in serv_addr;
-  struct sockaddr_in udp_cli_addr;	/* client's Internet socket
-					 * addr */
-  struct servent *sp = NULL;
   int verbose = FALSE;
-  char *server_address;
-  u_int addr;
-  struct in_addr *ptr;
   int n, nr = 0;
-  int rc;
 #ifdef OPENSSL
   int sslcode;
   char rand_file[MAXLINE];
@@ -89,16 +90,15 @@ main (argc, argv)
   unsigned short timeout_requested = 0;
   unsigned short size_requested = 0;
   char *url = "";
-  short port = 0;
-  char *text_port = malloc (6);
 #if USE_SIGACTION
   struct sigaction mysigaction;
 #endif
 
-  char *port_name = ECHO_TCP_PORT;
+  char port_name[NI_MAXSERV];
   unsigned short port_to_use = USE_ECHO;
   unsigned short http = 0;
   unsigned short smtp = 0;
+  unsigned short discard = 0;
   unsigned short udp = 0;
   unsigned short icp = 0;
 
@@ -124,7 +124,7 @@ main (argc, argv)
   int priority_requested = 0;
   int tos;
   int tos_requested = 0;
-  char *arg_end;
+  char *arg_end, *p;
 
   null_timeval.tv_sec = 0;
   null_timeval.tv_usec = 0;
@@ -137,15 +137,18 @@ main (argc, argv)
   median = null_timeval;
   max = null_timeval;
   min = max_timeval;
+  stddev = null_timeval;
+  strcpy (port_name, ECHO_TCP_PORT);
 
   for (i = 0; i <= MAXNUMBER; i++)
     {
       results[i].valid = 0;
     }
   progname = argv[0];
-  while ((ch = getopt (argc, argv, "vs:n:w:dch:i:rut:f:SCp:P:aA")) != EOF)
+  while ((result =
+	  getopt (argc, argv, "vs:n:w:dch:i:rut:f:SCp:P:aA46")) != -1)
     {
-      switch (ch)
+      switch ((char) result)
 	{
 	case 'v':
 	  verbose = TRUE;
@@ -160,23 +163,24 @@ main (argc, argv)
 	  ssl = 1;
 	  break;
 	case 'd':
-	  port_name = DISCARD_TCP_PORT;
+	  strcpy (port_name, DISCARD_TCP_PORT);
 	  port_to_use = USE_DISCARD;
+	  discard = 1;
 	  break;
 	case 'c':
-	  port_name = CHARACTER_GENERATOR_TCP_PORT;
+	  strcpy (port_name, CHARACTER_GENERATOR_TCP_PORT);
 	  port_to_use = USE_CHARGEN;
 	  stop_at_newlines = 0;
 	  break;
 	case 'i':
-	  port_name = DEFAULT_ICP_UDP_PORT;
+	  strcpy (port_name, DEFAULT_ICP_UDP_PORT);
 	  port_to_use = USE_ICP;
 	  udp = 1;
 	  icp = 1;
 	  url = optarg;
 	  break;
 	case 'h':
-	  port_name = DEFAULT_HTTP_TCP_PORT;
+	  strcpy (port_name, DEFAULT_HTTP_TCP_PORT);
 	  port_to_use = USE_HTTP;
 	  http = 1;
 	  url = optarg;
@@ -192,7 +196,7 @@ main (argc, argv)
 	  fill_requested = 1;
 	  break;
 	case 'S':
-	  port_name = "smtp";
+	  strcpy (port_name, "smtp");
 	  port_to_use = USE_SMTP;
 	  smtp = 1;
 	  break;
@@ -281,6 +285,12 @@ main (argc, argv)
 	      exit (1);
 	    }
 	  break;
+	case '4':
+	  family = AF_INET;
+	  break;
+	case '6':
+	  family = AF_INET6;
+	  break;
 	default:
 	  usage ();
 	}
@@ -372,7 +382,7 @@ main (argc, argv)
     }
   if (ssl && http)
     {
-      port_name = DEFAULT_HTTPS_TCP_PORT;
+      strcpy (port_name, DEFAULT_HTTPS_TCP_PORT);
     }
 #ifndef USE_TOS
   if (tos_requested)
@@ -407,88 +417,120 @@ main (argc, argv)
       printf ("\nThis is %s, version %s.\n\n", progname, VERSION);
     }
   server = argv[0];
+  if (!http && !icp)
+    {
+      for (p = server; *p && (*p != ':'); p++)
+	{
+	}
+      if (*p && (*p == ':'))
+	{
+	  (void) fprintf (stderr,
+			  "%s: The syntax hostname:port is only for HTTP or ICP\n",
+			  progname);
+	  exit (1);
+	}
+    }
+  signal (SIGINT, interrupted);
+  memset (&hints, 0, sizeof (hints));
+  hints.ai_family = family;
+  hints.ai_socktype = udp ? SOCK_DGRAM : SOCK_STREAM;
+
 #ifdef HTTP
   if (http || icp)
     {
-      if (icp)
-	{
-	  find_server_and_port (server, &port, port_name);
-	  if (port == 0)
-	    port = 3130;
+      char *text_port = NULL;
+
+      if (*server == '[')
+	{			/* RFC 2732 */
+	  server++;
+	  for (p = server; *p && *p != ']'; p++)
+	    {
+	    }
+	  p++;
+	  if (*p == ':')
+	    {
+	      p--;
+	      *p = 0;
+	      text_port = p + 2;
+	      strncpy (port_name, text_port, NI_MAXSERV);
+	    }
+	  else
+	    {
+	      p--;
+	      if (*p == ']')
+		{
+		  /* No port number */
+		  *p = 0;
+		}
+	      else
+		{
+		  (void) fprintf (stderr,
+				  "%s: Invalid syntax for IPv6 address.\n",
+				  progname);
+		  exit (1);
+		}
+	    }
 	}
       else
 	{
-	  find_server_and_port (server, &port, port_name);
-	  if (port == 0)
-	    port = 80;
+	  for (p = server; *p; p++)
+	    {
+	      if (*p == ':')
+		{
+		  *p = 0;
+		  text_port = p + 1;
+		  strncpy (port_name, text_port, NI_MAXSERV);
+		}
+	    }
+	}
+      if (text_port == NULL)
+	{
+	  error = getaddrinfo (server, port_name, &hints, &res);
+	  if (error)
+	    {
+	      if (error == EAI_SERVICE)
+		{
+		  if (strcmp (port_name, DEFAULT_HTTP_TCP_PORT) == 0)
+		    {
+		      strcpy (port_name, "80");
+		    }
+		  else if (strcmp (port_name, DEFAULT_HTTPS_TCP_PORT) == 0)
+		    {
+		      strcpy (port_name, "443");
+		    }
+		  else if (strcmp (port_name, DEFAULT_ICP_UDP_PORT) == 0)
+		    {
+		      strcpy (port_name, "3130");
+		    }
+		}
+	    }
 	}
-
-      sprintf (text_port, "(port %d)", ntohs (port));
     }
 #endif
-  signal (SIGINT, interrupted);
-  if ((addr = inet_addr (server)) == INADDR_NONE)
+
+  error = getaddrinfo (server, port_name, &hints, &res);
+  if (error)
     {
-      if ((hostptr = gethostbyname (server)) == NULL)
-	{
-	  err_quit ("gethostbyname error for host: %s %s",
-		    server, sys_err_str ());
-	}
-      server_address = *(hostptr->h_addr_list);	/* First item of the
-						 * list */
-      /*
-       * addr = (u_long) *server_address; 
-       */
-      /* ptr.s_addr = addr; */
-      ptr = (struct in_addr *) server_address;	/* hostptr->h_addr_list
-						 * points actually to
-						 * u_longs, not strings */
-      addr = ptr->s_addr;
+      err_quit ("getaddrinfo error for host: %s %s",
+		server, gai_strerror (error));
     }
-  else
+
+  if (getnameinfo (res->ai_addr, res->ai_addrlen, hbuf, sizeof (hbuf),
+		   pbuf, sizeof (pbuf), niflags) != 0)
     {
-      ptr = (struct in_addr *) malloc (sizeof (struct in_addr));
-      ptr->s_addr = addr;
-    }
-  if (!http && !icp)		/* Already find */
-    {
-      if (!udp)
-	{
-	  if ((sp = getservbyname (port_name, "tcp")) == NULL)
-	    {
-	      err_quit ("tcp_open: unknown service: %s/tcp", port_name);
-	    }
-	}
-      else
-	{
-	  if ((sp = getservbyname (port_name, "udp")) == NULL)
-	    {
-	      err_quit ("tcp_open: unknown service: %s/udp", port_name);
-	    }
-	}
+      strcpy (hbuf, "?");
+      strcpy (pbuf, "?");
     }
+
   /*
    * Fill in the structure "serv_addr" with the address of the server
    * that we want to connect with.
    */
 
-  bzero ((char *) &serv_addr, sizeof (serv_addr));
-  serv_addr.sin_family = AF_INET;
-  serv_addr.sin_addr.s_addr = addr;
-  if (!http && !icp)
-    {
-      serv_addr.sin_port = sp->s_port;
-    }
-  else
-    {
-      serv_addr.sin_port = port;
-    }
-
 #ifdef HTTP
   if (http)
     {
-      sendline =
-	make_http_sendline (url, server, (int) ntohs (port), nocache);
+      sendline = make_http_sendline (url, server, res->ai_protocol, nocache);
     }
   else
 #endif
@@ -504,7 +546,26 @@ main (argc, argv)
 #ifdef ICP
   if (icp)
     {
-      sendline = make_icp_sendline (url, &addr, opcode, &length);
+      if (res->ai_family == AF_INET)
+	{
+	  sendline =
+	    make_icp_sendline (url, &(res->ai_addr), opcode, &length);
+	}
+      else
+	{
+
+	  /* TODO: we should be able to create a ICP hostid for IPv6 addresses... 
+	     See the Squid IPv6 patch at 
+	     http://devel.squid-cache.org/projects.html#ipv6, for instance the 
+	     following code. */
+	  sendline = make_icp_sendline (url, (void *) NULL, opcode, &length);
+	  /*
+	     -    headerp->shostid = theOutICPAddr.s_addr;
+	     +    ** FIXME ** we should get more unique data from IPv6 address 
+	     +xmemcpy (&headerp->shostid, &theOutICPAddr,
+	     sizeof (headerp->shostid));
+	   */
+	}
     }
   else
 #endif
@@ -538,7 +599,14 @@ main (argc, argv)
 
   for (i = 1; i <= number; i++)
     {
-
+      if (i > 1)
+	{
+#ifdef HAVE_USLEEP
+	  usleep (wait * 1000000);
+#else
+	  sleep (wait);
+#endif
+	}
       attempts++;
 #ifdef OPENSSL
       if (ssl)
@@ -550,26 +618,26 @@ main (argc, argv)
 	if ((sslh = SSL_new (ctx)) == NULL)
 	  err_sys ("Cannot initialize SSL context");
 #endif
-      if (!udp)
+      /*
+       * Open a socket.
+       */
+      if ((sockfd =
+	   socket (res->ai_family, res->ai_socktype, res->ai_protocol)) < 0)
+	err_sys ("Can't open socket");
+      if (udp)
 	{
-	  /*
-	   * Open a TCP socket (an Internet stream socket).
-	   */
+	  struct addrinfo hints2, *res2;
 
-	  if ((sockfd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
-	    err_sys ("Can't open stream socket");
-	}
-      else
-	{
-	  if ((sockfd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
-	    err_sys ("Can't open datagram socket");
-	  /* Bind socket for reply. Not necessary? */
-	  bzero ((char *) &udp_cli_addr, sizeof (udp_cli_addr));
-	  udp_cli_addr.sin_family = AF_INET;
-	  udp_cli_addr.sin_addr.s_addr = htonl (INADDR_ANY);
-	  udp_cli_addr.sin_port = htons (0);
-	  if (bind (sockfd, (struct sockaddr *) &udp_cli_addr,
-		    sizeof (udp_cli_addr)) < 0)
+	  memset (&hints2, 0, sizeof (hints2));
+	  hints2.ai_family = res->ai_family;
+	  hints2.ai_flags = AI_PASSIVE;
+	  hints2.ai_socktype = SOCK_DGRAM;
+	  error = getaddrinfo (NULL, "0", &hints2, &res2);
+	  if (error)
+	    {
+	      err_sys ("getaddrinfo error");
+	    }
+	  if (bind (sockfd, res2->ai_addr, res2->ai_addrlen) < 0)
 	    {
 	      err_sys ("bind error");
 	    }
@@ -613,20 +681,20 @@ main (argc, argv)
 	    {
 	      printf
 		("Trying to connect to internet address %s %s to transmit %u bytes...\n",
-		 inet_ntoa (*ptr), (port == 0 ? "" : text_port), n);
+		 hbuf, pbuf, n);
 	    }
 #ifdef ICP
 	  if (icp)
 	    {
 	      printf
 		("Trying to send an ICP packet of %u bytes to the internet address %s...\n",
-		 length, inet_ntoa (*ptr));
+		 length, hbuf);
 	    }
 #endif
 	  else
 	    {
 	      printf ("Trying to send %u bytes to internet address %s...\n",
-		      size, inet_ntoa (*ptr));
+		      size, hbuf);
 	    }
 #ifdef FLUSH_OUTPUT
 	  if (fflush ((FILE *) NULL) != 0)
@@ -660,13 +728,13 @@ main (argc, argv)
 	  /*
 	   * Connect to the server.
 	   */
-
-	  if (connect (sockfd, (struct sockaddr *) &serv_addr,
-		       sizeof (serv_addr)) < 0)
+	  (void) gettimeofday (&conntv, (struct timezone *) NULL);
+	  if (connect (sockfd, res->ai_addr, res->ai_addrlen) < 0)
 	    {
 	      if ((errno == EINTR) && (timeout_flag))
 		{
 		  printf ("Timeout while connecting\n");
+		  close (sockfd);
 		  continue;
 #ifdef FLUSH_OUTPUT
 		  if (fflush ((FILE *) NULL) != 0)
@@ -678,9 +746,25 @@ main (argc, argv)
 	      else
 		err_sys ("Can't connect to server");
 	    }
+	  else
+	    {
+	      if (tcp)
+		{
+		  (void) gettimeofday (&connectedtv,
+				       (struct timezone *) NULL);
+		  temp = connectedtv;
+		  tvsub (&temp, &conntv);
+		  if (verbose)
+		    {
+		      printf ("Connected...\n");
+		      printf ("TCP Latency: %d.%06d seconds\n",
+			      (int) temp.tv_sec, (int) temp.tv_usec);
+		    }
+		}
+	    }
+
 	  if (verbose && tcp)
 	    {
-	      printf ("Connected...\n");
 #ifdef FLUSH_OUTPUT
 	      if (fflush ((FILE *) NULL) != 0)
 		{
@@ -699,6 +783,7 @@ main (argc, argv)
 		if ((errno == EINTR) && (timeout_flag))
 		  {
 		    printf ("Timeout while starting SSL\n");
+		    close (sockfd);
 		    continue;
 		  }
 	      if (verbose)
@@ -721,8 +806,7 @@ main (argc, argv)
 	  if (ttcp)
 	    {
 	      if (sendto (sockfd, sendline, n, MSG_EOF,
-			  (struct sockaddr *) &serv_addr,
-			  sizeof (serv_addr)) != n)
+			  res->ai_addr, res->ai_addrlen) != n)
 		err_sys ("sendto error on socket");
 	      if (verbose)
 		{
@@ -742,6 +826,7 @@ main (argc, argv)
 			{
 			  nr = n;
 			  printf ("Timeout while writing\n");
+			  close (sockfd);
 			  continue;
 			}
 		      else
@@ -757,6 +842,7 @@ main (argc, argv)
 			{
 			  nr = n;
 			  printf ("Timeout while writing\n");
+			  close (sockfd);
 			  continue;
 			}
 		      else
@@ -766,6 +852,7 @@ main (argc, argv)
 				   ERR_error_string (sslcode, NULL));
 			}
 		    }
+		  /* printf ("DEBUG: writing %s with SSL\n", sendline); */
 		}
 #endif
 	      /* Write something to the server */
@@ -775,6 +862,7 @@ main (argc, argv)
 		    {
 		      nr = n;
 		      printf ("Timeout while writing\n");
+		      close (sockfd);
 		      continue;
 		    }
 		  else
@@ -787,7 +875,7 @@ main (argc, argv)
 	      if (icp)
 		{
 		  if (sendto (sockfd, sendline, length, 0,
-			      &serv_addr, sizeof (serv_addr)) != length)
+			      res->ai_addr, res->ai_addrlen) != length)
 		    err_sys ("sendto error on socket");
 		}
 	      else
@@ -802,6 +890,7 @@ main (argc, argv)
 	    }
 	  if (verbose)
 	    {
+	      (void) gettimeofday (&sendtv, (struct timezone *) NULL);
 #ifdef ICP
 	      if (icp)
 		printf ("Sent (%d bytes)...\n", length);
@@ -817,17 +906,51 @@ main (argc, argv)
 #endif
 	    }
 	}
+
+      if (tcp && !discard)
+	{
+	  fd_set mask;
+	  int n = 0;
+
+	  FD_ZERO (&mask);
+
+	  if (!(http && ssl))
+	    n = fileno (files);
+#ifdef OPENSSL
+	  else
+	    {
+	      n = SSL_get_fd (sslh);
+	    }
+#endif
+
+	  FD_SET (n, &mask);
+	  if (select (n + 1, &mask, 0, 0, NULL) > 0)
+	    {
+	      (void) gettimeofday (&recvtv, (struct timezone *) NULL);
+	      temp = recvtv;
+	      tvsub (&temp, &sendtv);
+	      if (verbose)
+		printf ("Application Latency: %d.%06d seconds\n",
+			(int) temp.tv_sec, (int) temp.tv_usec);
+	    }
+
+	}
+
       if ((port_to_use == USE_ECHO) || (port_to_use == USE_CHARGEN) ||
 	  (port_to_use == USE_HTTP) || (port_to_use == USE_ICP) ||
 	  (port_to_use == USE_SMTP))
 	{
 	  if (!udp)
 	    {
-	      if (!http && !smtp)
+	      if (!http && !smtp && !discard)
 		{
 		  /* Read from the server */
 		  nr = readline (files, recvline, n, stop_at_newlines);
 		}
+	      else if (discard)
+		{
+		  /* No reply, no read */
+		}
 #ifdef HTTP
 	      else if (http)
 		{
@@ -846,6 +969,8 @@ main (argc, argv)
 		  nr = smtp_read_response_from_server (files);
 		}
 #endif
+
+
 	    }
 	  else
 	    {
@@ -903,7 +1028,7 @@ main (argc, argv)
 		}
 #endif
 	    }
-	  if (!http && !icp && !smtp)
+	  if (!http && !icp && !smtp && !discard)
 	    {
 	      if ((nr < 0 || nr != n) && timeout_flag)
 		/* if ((nr < 0 || nr != n) && (errno == EINTR) && timeout_flag) */
@@ -917,6 +1042,7 @@ main (argc, argv)
 		      err_sys ("I cannot flush");
 		    }
 #endif
+		  close (sockfd);
 		  continue;
 		}
 	      if (nr < 0 || nr != n)
@@ -924,7 +1050,7 @@ main (argc, argv)
 			 nr, n);
 	    }
 	  else
-	    /* This is HTTP */
+	    /* This is probably HTTP */
 	    {
 	      if ((nr < 0) && (errno == EINTR) && (timeout_flag))
 		{
@@ -936,6 +1062,7 @@ main (argc, argv)
 		      err_sys ("I cannot flush");
 		    }
 #endif
+		  close (sockfd);
 		  continue;
 		}
 	      if (nr < 0)
@@ -947,6 +1074,8 @@ main (argc, argv)
 	    printf ("%d bytes read from server.\n", nr);
 	}
       /* That's all, folks */
+      if (tcp)
+	alarm (0);
       if (http)
 	{
 #ifdef OPENSSL
@@ -959,8 +1088,6 @@ main (argc, argv)
       close (sockfd);
 
       (void) gettimeofday (&newtv, (struct timezone *) NULL);
-      if (tcp)
-	alarm (0);
       temp = newtv;
       tvsub (&temp, &oldtv);
       if (!timeout_flag)
@@ -993,7 +1120,7 @@ main (argc, argv)
 	      recvline[strlen (sendline)] = 0;
 	      if (strcmp (sendline, recvline) != 0)
 		{
-		  /* Todo: it does not work if the size is lower than the
+		  /* TODO: it does not work if the size is lower than the
 		     length of CHARGENERATED */
 		  printf (" I got back:\n%s\n", recvline);
 		  printf (" instead of the most common:\n%s\n", sendline);
@@ -1027,11 +1154,6 @@ main (argc, argv)
 	      /* SSL_clear (sslh); No, we have to free. Bug #130151 */
 	      SSL_free (sslh);
 	    }
-#endif
-#ifdef HAVE_USLEEP
-	  usleep (wait * 1000000);
-#else
-	  sleep (wait);
 #endif
 	}
     }
@@ -1070,6 +1192,9 @@ printstats ()
       /* The number of bytes/second, as printed above, is not really
          meaningful: size does not reflect the number of bytes exchanged.
          With echo, N = 2*size, with discard, N = size, with http, N = size + (response)... */
+      tvstddev (&stddev, successes, total, results);
+      printf ("Standard deviation: %d.%06d\n",
+	      (int) stddev.tv_sec, (int) stddev.tv_usec);
       for (i = 0; i < number; i++)
 	{
 	  if (results[i].valid)
@@ -1101,7 +1226,7 @@ printstats ()
 	  tvadd (&median, &good_results[successes / 2]);
 	  tvavg (&median, 2);
 	}
-      printf ("Median  time: %d.%06d seconds (%.0f bytes per sec.)\n",
+      printf ("Median time: %d.%06d seconds (%.0f bytes per sec.)\n",
 	      (int) median.tv_sec, (int) median.tv_usec,
 	      (double) size / tv2double (median));
     }
