@@ -1,6 +1,6 @@
 #include "echoping.h"
 
-/* $Id: error.c,v 4.4 2001/02/23 10:41:22 bortz Exp $ */
+/* $Id: error.c,v 4.5 2002/09/25 13:00:29 bortz Exp $ */
 
 /* Most of error-handling routines stolen from Stevens' books */
 
@@ -88,7 +88,7 @@ void
 usage ()
 {
   fprintf (stderr,
-	   "Usage: %s [-v] [-r] [-f fill] [-t timeout] [-c] [-d] [-u] [-s size] [-n number] [-w delay] [-h url] [-i url] [-p priority] [-P tos] [-C] [-S] hostname[:port]\n",
+	   "Usage: %s [-4] [-6] [-v] [-r] [-f fill] [-t timeout] [-c] [-d] [-u] [-s size] [-n number] [-w delay] [-h url] [-i url] [-p priority] [-P tos] [-C] [-S] hostname[:port]\n",
 	   progname);
   exit (1);
 }
