@@ -1,4 +1,4 @@
-/* $Id: http.c,v 4.3 2001/03/11 13:38:45 bortz Exp $ */
+/* $Id: http.c,v 4.6 2002/10/03 13:53:23 bortz Exp $ */
 
 #include "echoping.h"
 
@@ -49,65 +49,6 @@ make_http_sendline (char *url, char *host, int port, int nocache)
   return sendline;
 }
 
-void
-find_server_and_port (char *server, short *port, char *default_port)
-{
-  char *text_port, *p;
-  struct servent *sp;
-  for (p = server; *p; p++)
-    {
-      if (*p == ':')
-	{
-	  *p = 0;
-	  text_port = p + 1;
-	  *port = atoi (text_port);
-	}
-    }
-  if (*port == 0)
-    {
-      if (strcmp (default_port, DEFAULT_HTTP_TCP_PORT) == 0)
-	{
-	  if ((sp = getservbyname ("http", "tcp")) == NULL)
-	    {
-	      if ((sp = getservbyname ("www", "tcp")) == NULL)
-		{
-		  *port = htons (80);
-		  return;
-		}
-	    }
-	  *port = sp->s_port;
-	  return;
-	}
-      else if (strcmp (default_port, DEFAULT_HTTPS_TCP_PORT) == 0)
-	{
-	  if ((sp = getservbyname ("https", "tcp")) == NULL)
-	    {
-	      *port = htons (443);
-	      return;
-	    }
-	  *port = sp->s_port;
-	  return;
-	}
-      else if (strcmp (default_port, DEFAULT_ICP_UDP_PORT) == 0)
-	{
-	  if ((sp = getservbyname ("icp", "udp")) == NULL)
-	    {
-	      *port = htons (3130);
-	      return;
-	    }
-	  *port = sp->s_port;
-	  return;
-	}
-      else if ((sp = getservbyname (default_port, "tcp")) == NULL)
-	{
-	  err_quit ("tcp_open: unknown service: %s/tcp", default_port);
-	}
-      *port = sp->s_port;
-    }
-  else
-    *port = htons (*port);
-}
-
 int
 read_from_server (CHANNEL fs, short ssl)
 {
@@ -124,16 +65,18 @@ read_from_server (CHANNEL fs, short ssl)
       if (!ssl)
 	nr = readline (fs.fs, big_recvline, MAXTOREAD, TRUE);
 #ifdef OPENSSL
-      else {
-	nr = SSL_readline (fs.ssl, big_recvline, MAXTOREAD, TRUE);
-	if (nr == -1) {
-	  sslcode = ERR_get_error ();
-	  err_ret ("SSL_readline error: %s",
-		   ERR_error_string (sslcode, NULL));
+      else
+	{
+	  nr = SSL_readline (fs.ssl, big_recvline, MAXTOREAD, TRUE);
+	  if (nr == -1)
+	    {
+	      sslcode = ERR_get_error ();
+	      err_ret ("SSL_readline error: %s",
+		       ERR_error_string (sslcode, NULL));
+	    }
 	}
-      }
 #endif
-      /* printf ("DEBUG: reading \"%s\"\n (%d chars)\n", big_recvline, nr); */
+      /* printf ("DEBUG: reading \"%s\"\n (%d chars)\n", big_recvline, nr);  */
       /* HTTP replies should be separated by CR-LF. Unfortunately, some
          servers send only CR :-( */
       body = ((nr == 2) || (nr == 1));	/* Empty line CR-LF seen */
